import Flutter
import UIKit
import Stripe
import RxSwift

public class SwiftFlutterStripePlugin: NSObject {
    
    private lazy var disposeBag = DisposeBag()
    
    private var transactionToken: String?
    private var handlingNativePayResult: FlutterResult?
}

extension SwiftFlutterStripePlugin: FlutterPlugin {
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "flutter_stripe", binaryMessenger: registrar.messenger())
        let instance = SwiftFlutterStripePlugin()
        STPPaymentConfiguration.shared().publishableKey = "pk_test_pHzj1pASIO4PwqVZ7LtKfxuR"
        registrar.addMethodCallDelegate(instance, channel: channel)
    }
    
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        guard let type = FlutterStripeAPIType(rawValue: call.method) else {
            result(FlutterMethodNotImplemented)
            return
        }
        switch type {
        case .getPlatformVersion:
            getPlatformVersion(result)
        case .isNativePaySupport:
            isNatativePaySupport(result)
        case .promptNativePayment:
            promptNativePayment(call, result)
        }
    }
}

extension SwiftFlutterStripePlugin: FlutterStripeAPI {
    
    func submitStripeTokenToServer(_ token: STPToken, _ completion: @escaping (ErrorMessage?) -> Void) {
        RxRequest
            .submitAction(PaymentRouter.submit(token: transactionToken ?? ""))
            .do(onNext: { _ in
                completion(nil)
            }, onError: {
                completion( ($0 as? RxRequestError)?.localizedDescription ?? "Unknown Error" )
            })
            .catchByEmpty()
            .subscribe()
            .disposed(by: disposeBag)
    }
    
    
    func promptNativePayment(_ call: FlutterMethodCall, _ result: @escaping (Any?) -> Void) {
        guard let item = (call.arguments as? [String: Double])?.first else {
            result(FlutterError(code: FlutterStripeError.wrongParamenters.rawValue, message: "Paramenter format: [String: Double]", details: nil))
            return
        }
        
        let request = Stripe.paymentRequest(withMerchantIdentifier: "merchant.com.hkmovie6.flutterStripeExample", country: "HK", currency: "HKD")
        self.transactionToken = item.key
        request.paymentSummaryItems = [PKPaymentSummaryItem(label: "EM", amount: NSDecimalNumber(value: item.value))]
        
        guard let vc = PKPaymentAuthorizationViewController(paymentRequest: request), Stripe.canSubmitPaymentRequest(request) else {
            result(FlutterError(code: FlutterStripeError.submitNativePayError.rawValue, message: "Cannot submit native pay via stripe", details: nil))
            return
        }
        
        self.handlingNativePayResult = result
        vc.delegate = self
        UIApplication.presentingVC()?.present(vc, animated: true)
    }
    
    func isNatativePaySupport(_ result: (Any?) -> Void) {
        result(Stripe.deviceSupportsApplePay())
    }
    
    func getPlatformVersion(_ result: (Any?) -> Void) {
        result("iOS \(UIDevice.current.systemVersion)")
    }
}

extension SwiftFlutterStripePlugin: PKPaymentAuthorizationViewControllerDelegate {
    @available(iOS 11.0, *)
    public func paymentAuthorizationViewController(_ controller: PKPaymentAuthorizationViewController, didAuthorizePayment payment: PKPayment, handler completion: @escaping (PKPaymentAuthorizationResult) -> Void) {
        STPAPIClient.shared().createToken(with: payment) { [weak self] (token: STPToken?, error: Error?) in
            guard let token = token, error == nil else {
                self?.handlingNativePayResult?(FlutterError(code: FlutterStripeError.createNativePayError.rawValue, message: error!.localizedDescription, details: error))
                return
            }
            
            self?.submitStripeTokenToServer(token) { [weak self] errMsg in
                let succeed = errMsg == nil
                self?.handlingNativePayResult?(succeed ? token.jsonString : FlutterError(code: FlutterStripeError.submitNativePayError.rawValue, message: errMsg, details: error))
                completion(PKPaymentAuthorizationResult(status: succeed ? .success : .failure, errors: nil))
            }
        }
    }
    
    public func paymentAuthorizationViewController(_ controller: PKPaymentAuthorizationViewController, didAuthorizePayment payment: PKPayment, completion: @escaping (PKPaymentAuthorizationStatus) -> Void) {
        STPAPIClient.shared().createToken(with: payment) { [weak self] (token: STPToken?, error: Error?) in
            guard let token = token, error == nil else {
                self?.handlingNativePayResult?(FlutterError(code: FlutterStripeError.createNativePayError.rawValue, message: error!.localizedDescription, details: error))
                return
            }
            
            self?.submitStripeTokenToServer(token) { [weak self] errMsg in
                let succeed = errMsg == nil
                self?.handlingNativePayResult?(errMsg == nil ? token.jsonString : FlutterError(code: FlutterStripeError.submitNativePayError.rawValue, message: errMsg, details: error))
                completion(succeed ? .success : .failure)
            }
        }
    }
    
    public func paymentAuthorizationViewControllerDidFinish(_ controller: PKPaymentAuthorizationViewController) {
        controller.dismiss(animated: true)
        self.handlingNativePayResult = nil
    }
    
}
